import { renderHook, act } from '@testing-library/react';
import { useAtom } from 'jotai';
import { coordinateSystemAtom, relativeCoordinateSystemAtom } from './Map';

describe('The Map component coordinate system', () => {
  test('is initialized', () => {
    const { result } = renderHook(() => useAtom(coordinateSystemAtom));
    expect(result.current[0]).toMatchInlineSnapshot(`
Object {
  "shift": Object {
    "x": 0,
    "y": 0,
  },
  "zoom": 1,
}
`);
  });

  test('reacts to a translation', () => {
    const { result } = renderHook(() => [
      useAtom(coordinateSystemAtom),
      useAtom(relativeCoordinateSystemAtom),
    ]);
    act(() => {
      result.current[0][1]({
        shift: {
          x: 0,
          y: 0,
        },
        zoom: 1,
      } as any);
      result.current[1][1]({
        deltaShift: { x: 5, y: 1 },
        zoomCenter: null,
        deltaZoom: null,
      } as any);
    });
    expect(result.current[0][0]).toMatchInlineSnapshot(`
Object {
  "shift": Object {
    "x": 5,
    "y": 1,
  },
  "zoom": 1,
}
`);
  });

  test('reacts to a zoom on its origin', () => {
    const { result } = renderHook(() => [
      useAtom(coordinateSystemAtom),
      useAtom(relativeCoordinateSystemAtom),
    ]);
    act(() => {
      result.current[0][1]({
        shift: {
          x: 0,
          y: 0,
        },
        zoom: 1,
      } as any);
      result.current[1][1]({
        deltaShift: null,
        zoomCenter: null,
        deltaZoom: 5,
      } as any);
    });
    expect(result.current[0][0]).toMatchInlineSnapshot(`
Object {
  "shift": Object {
    "x": 0,
    "y": 0,
  },
  "zoom": 5,
}
`);
  });

  test('reacts to a zoom around another point', () => {
    const { result } = renderHook(() => [
      useAtom(coordinateSystemAtom),
      useAtom(relativeCoordinateSystemAtom),
    ]);
    act(() => {
      result.current[0][1]({
        shift: {
          x: 0,
          y: 0,
        },
        zoom: 1,
      } as any);
      result.current[1][1]({
        deltaShift: null,
        zoomCenter: { x: 2, y: 2 },
        deltaZoom: 5,
      } as any);
    });
    expect(result.current[0][0]).toMatchInlineSnapshot(`
Object {
  "shift": Object {
    "x": -8,
    "y": -8,
  },
  "zoom": 5,
}
`);
  });

  test('reacts to a zoom (around another point) and a translation', () => {
    const { result } = renderHook(() => [
      useAtom(coordinateSystemAtom),
      useAtom(relativeCoordinateSystemAtom),
    ]);
    act(() => {
      result.current[0][1]({
        shift: {
          x: 0,
          y: 0,
        },
        zoom: 1,
      } as any);
      result.current[1][1]({
        deltaShift: { x: 1, y: 3 },
        zoomCenter: { x: 2, y: 2 },
        deltaZoom: 5,
      } as any);
    });
    expect(result.current[0][0]).toMatchInlineSnapshot(`
Object {
  "shift": Object {
    "x": -7,
    "y": -5,
  },
  "zoom": 5,
}
`);
  });
});
