import Tile from './Tile';
import { TileFactory, TileKeyObject } from './types';

export interface TileProvider {
  name: string;
  minZoom: number;
  maxZoom: number;
  getTileUrl: { (zoom: number, x: number, y: number): string };
}

const getRandomItem = (items: any[]) => {
  const idx = Math.floor(Math.random() * items.length);
  return items[idx];
};

const abc = ['a', 'b', 'c'];

const tileProviders: any = {
  osm: {
    name: 'Open Street Map',
    minZoom: 0,
    maxZoom: 19,
    getTileUrl: (zoom: number, x: number, y: number) =>
      'https://tile.openstreetmap.org/' + zoom + '/' + x + '/' + y + '.png',
  },
  osmfr: {
    name: 'Open Street Map France',
    minZoom: 0,
    maxZoom: 20,
    getTileUrl: (zoom: number, x: number, y: number) =>
      'https://' +
      getRandomItem(abc) +
      '.tile.openstreetmap.fr/osmfr/' +
      zoom +
      '/' +
      x +
      '/' +
      y +
      '.png',
  },
  otm: {
    name: 'Open Topo Map',
    minZoom: 2,
    maxZoom: 17,
    getTileUrl: (zoom: number, x: number, y: number) =>
      'https://' +
      getRandomItem(abc) +
      '.tile.opentopomap.org/' +
      zoom +
      '/' +
      x +
      '/' +
      y +
      '.png',
  },
  cyclosm: {
    name: 'CyclOSM',
    minZoom: 0,
    maxZoom: 19,
    getTileUrl: (zoom: number, x: number, y: number) =>
      'https://' +
      getRandomItem(abc) +
      '.tile-cyclosm.openstreetmap.fr/cyclosm/' +
      zoom +
      '/' +
      x +
      '/' +
      y +
      '.png',
  },
  //https://b.tile.openstreetmap.fr/openriverboatmap/20/535762/382966.png
  openriverboatmap: {
    name: 'Open River Boat Map',
    minZoom: 0,
    maxZoom: 20,
    getTileUrl: (zoom: number, x: number, y: number) =>
      'https://' +
      getRandomItem(abc) +
      '.tile.openstreetmap.fr/openriverboatmap/' +
      zoom +
      '/' +
      x +
      '/' +
      y +
      '.png',
  },

  // cyclosmlite: {
  //   name: 'CyclOSM lite',
  //   minZoom: 0,
  //   maxZoom: 19,
  //   getTileUrl: (zoom: number, x: number, y: number) =>
  //     'https://' +
  //     getRandomItem(abc) +
  //     '.tile-cyclosm.openstreetmap.fr/cyclosm-lite/' +
  //     zoom +
  //     '/' +
  //     x +
  //     '/' +
  //     y +
  //     '.png',
  // },
  // esrisat: {
  //   name: 'ESRI Satellite',
  //   minZoom: 0,
  //   maxZoom: 19,
  //   getTileUrl: (zoom: number, x: number, y: number) =>
  //     'https://server.arcgisonline.com/ArcGIS/rest/services/World_Imagery/MapServer/tile/' +
  //     zoom +
  //     '/' +
  //     x +
  //     '/' +
  //     y +
  //     '.jpg',
  // },

  fake: {
    name: 'Fake provider',
    minZoom: 0,
    maxZoom: 20,
    getTileUrl: (zoom: number, x: number, y: number) =>
      'https://fakeurl/' + zoom + '/' + x + '/' + y + '.png',
  },
};

const mod = (n: number, m: number) => {
  const jsMod = n % m;
  return jsMod >= 0 ? jsMod : jsMod + m;
};

/**
 *
 * @param keyObject The tile identifier
 * @returns The `<Tile>`'s URL 'as a string)
 */
export const getTileUrl = (keyObject: TileKeyObject) => {
  const nbTiles = 2 ** keyObject.zoomLevel;
  const x = mod(keyObject.x, nbTiles);
  const y = mod(keyObject.y, nbTiles);
  return tileProviders[keyObject.provider].getTileUrl(
    keyObject.zoomLevel,
    x,
    y
  );
};
export default getTileUrl;
